package dammfine.dict;

import java.net.Socket;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;

import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.StringTokenizer;

import dammfine.dict.receiving.*;
import dammfine.dict.sending.*;

/**
 * <p>Provides a non-persistent connection to a dict server.</p>
 * 
 * <p>Example of use:
 * <pre>
 * DictConnection conn = new DictConnection( socket );
 *
 * conn.sendCommand( new ClientCommand() );
 * List receivedCommands = conn.sendCommand( new ShowDatabasesCommand() );
 * conn.sendCommand( new DefineCommand( "*", "heat" ) );
 * conn.sendCommand( new QuitCommand() );
 * </pre>
 * </p>
 */
public class DictConnection
{
	private ReceivingCommandRegistry commands = new ReceivingCommandRegistry();

	private BufferedReader inputReader;
	private PrintWriter outputWriter;
	
	/**
	 * Pass an open socket to the server.
	 */
	public DictConnection( BufferedReader inputReader, PrintWriter outputWriter )
		throws IOException
	{
		this.inputReader = inputReader;
		this.outputWriter = outputWriter;

		// Get the 220 server OK command.
		readCommand();
	}

	public DictConnection( Socket socket )
		throws IOException
	{
		this( 
			new BufferedReader( new InputStreamReader( socket.getInputStream() ) ),
			new PrintWriter( socket.getOutputStream() ) );
	}
	
	/**
	 * Sends a command and returns the responses, in order.
	 */
	public List sendCommand( SendingCommand sendCmd )
		throws IOException
	{
		LinkedList responses = new LinkedList();
		
		String sendStr = sendCmd.render();
		System.out.println( "Sending: " + sendStr );
		outputWriter.print( sendStr + (char)13 + (char)10 );
		outputWriter.flush();
		
		ReceivingCommand rcvCmd;
		do
		{
			rcvCmd = readCommand();
			System.out.println( "Got response: " + rcvCmd );
			responses.add( rcvCmd );

			if( rcvCmd.isNegative() )
				break;
		}
		while( ! rcvCmd.isPositiveCompletion() && !rcvCmd.isNegative() );
		
		return responses;
	}

	/**
	 * Reads a command off the input stream.
	 */
	private ReceivingCommand readCommand()
		throws IOException
	{
		ReceivingCommand rcvCmd;
		System.out.println( "Reading." );
		String line = inputReader.readLine();
		System.out.println( "Got: " + line );
		StringTokenizer tokens = new StringTokenizer( line );
		String num = tokens.nextToken();
		Integer code = new Integer( num );
		
		System.out.println( "Getting cmd for code " + code );
		
		// Load the factory command.
		rcvCmd = commands.getCommand( code );

		if( rcvCmd == null )
		{
			System.out.println( "No command defined!" );
		}
		
		// Now get the real instance.
		rcvCmd = rcvCmd.parse( line, inputReader );
		
		return rcvCmd;
	}

	public static void main( String args[] )
		throws Exception
	{
		Socket socket = new Socket( args[0], Integer.parseInt( args[1]) );
		
		DictConnection conn = new DictConnection( socket );
		
		System.out.println( "Ok" );
		conn.sendCommand( new ClientCommand() );
		conn.sendCommand( new ShowDatabasesCommand() );
		conn.sendCommand( new DefineCommand( "*", "heat" ) );
		conn.sendCommand( new QuitCommand() );
		
	}
}
